import React from 'react'
import { get } from 'lodash'
import { connect } from 'react-redux'
import styled from 'styled-components'
import { th } from '@pubsweet/ui-toolkit'
import { actions } from 'pubsweet-client'
import { withJournal } from 'xpub-journal'
import { ConnectPage } from 'xpub-connect'
import { DragDropContext } from 'react-dnd'
import { Icon, Button } from '@pubsweet/ui'
import HTML5Backend from 'react-dnd-html5-backend'
import { selectCollection, selectFragment } from 'xpub-selectors'
import { withStateHandlers, compose, toClass, withProps } from 'recompose'

import {
  withModal,
  SubmissionModal,
} from 'pubsweet-component-modal/src/components'

import {
  reduxForm,
  getFormValues,
  hasSubmitFailed,
  getFormSyncErrors,
} from 'redux-form'

import { Steps, FormItems } from 'pubsweet-components-faraday/src/components'

import { wizardSteps } from './'
import { autosaveRequest } from '../redux/autosave'
import AutosaveIndicator from './AutosaveIndicator'
import { submitManuscript } from '../redux/conversion'
import { onChange, onSubmit, setInitialValues, validate } from './utils'

const { Row } = FormItems

// #region wizard
const NewWizard = ({
  step,
  history,
  prevStep,
  isLastStep,
  isFirstStep,
  handleSubmit,
  journal: { manuscriptTypes = [] },
  ...rest
}) => (
  <Root className="wizard-root">
    <Steps currentStep={step} margin="0 20px 60px 0">
      {wizardSteps.map(({ stepTitle }) => (
        <Steps.Step key={stepTitle} title={stepTitle} />
      ))}
    </Steps>

    <StepRoot className="wizard-step">
      <IconButton
        data-test="submission-close"
        onClick={() => history.push('/')}
      >
        <Icon primary size={4}>
          x
        </Icon>
      </IconButton>
      {wizardSteps[step].component({ manuscriptTypes, ...rest })}
      <Row centered>
        <ButtonContainer isFirstStep={isFirstStep}>
          {!isFirstStep && (
            <Button
              data-test="submission-back"
              onClick={prevStep}
            >{`< BACK`}</Button>
          )}
          <Button data-test="submission-next" onClick={handleSubmit} primary>
            {isLastStep ? `SUBMIT MANUSCRIPT` : `NEXT STEP >`}
          </Button>
        </ButtonContainer>
      </Row>
      <Auto>
        <AutosaveIndicator formName="submission" />
      </Auto>
    </StepRoot>
  </Root>
)
// #endregion

// #region export
export default compose(
  ConnectPage(({ match }) => [
    actions.getCollection({ id: match.params.project }),
    actions.getFragment(
      { id: match.params.project },
      { id: match.params.version },
    ),
  ]),
  withJournal,
  connect(
    (state, { match }) => ({
      formValues: getFormValues('submission')(state),
      submitFailed: hasSubmitFailed('submission')(state),
      formSyncErrors: getFormSyncErrors('submission')(state),
      version: selectFragment(state, get(match, 'params.version')),
      project: selectCollection(state, get(match, 'params.project')),
    }),
    {
      autosaveRequest,
      submitManuscript,
      updateFragment: actions.updateFragment,
    },
  ),
  withStateHandlers(
    { step: 0 },
    {
      nextStep: ({ step }) => () => ({
        step: Math.min(wizardSteps.length - 1, step + 1),
      }),
      prevStep: ({ step }) => () => ({ step: Math.max(0, step - 1) }),
    },
  ),
  withProps(setInitialValues),
  withProps(({ formValues, formSyncErrors, submitFailed, step }) => ({
    isFirstStep: step === 0,
    isLastStep: step === wizardSteps.length - 1,
    filesError: submitFailed && get(formSyncErrors, 'files', ''),
    authorsError: submitFailed && get(formSyncErrors, 'authors', ''),
    hasConflicts: get(formValues, 'conflicts.hasConflicts', 'no') === 'yes',
  })),
  withModal(() => ({
    modalComponent: SubmissionModal,
  })),
  reduxForm({
    form: 'submission',
    validate,
    onChange,
    onSubmit,
  }),
  DragDropContext(HTML5Backend),
  toClass,
)(NewWizard)
// #endregion

// #region styled-components
const Root = styled.div`
  margin: 0 auto;
  width: calc(${th('gridUnit')} * 40);
`

const Auto = styled.div`
  position: absolute;
  right: 5px;
  bottom: 5px;
`

const StepRoot = styled.div`
  align-items: flex-start;
  background-color: ${th('colorBackground')};
  border: ${th('borderDefault')};
  display: flex;
  flex-direction: column;
  padding: 30px;
  position: relative;
`

const IconButton = styled.button`
  align-items: center;
  background-color: transparent;
  border: none;
  color: ${th('colorPrimary')};
  cursor: ${({ hide }) => (hide ? 'auto' : 'pointer')};
  display: flex;
  font-family: ${th('fontInterface')};
  font-size: ${th('fontSizeBaseSmall')};
  opacity: ${({ hide }) => (hide ? 0 : 1)};
  text-align: left;

  position: absolute;
  top: 10px;
  right: 10px;

  &:active,
  &:focus {
    outline: none;
  }
  &:hover {
    opacity: 0.7;
  }
`

const ButtonContainer = styled.div`
  align-items: center;
  display: flex;
  flex-direction: row;
  justify-content: ${({ isFirstStep }) =>
    isFirstStep ? 'center' : 'space-between'};

  width: calc(${th('gridUnit')} * 14);
`
// #endregion
